/* ***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Initial Developer of the Original Code is
Malte Kraus.

Contributor(s):

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

/* Function getDescription()
 * Retrieves the localized description from the preferences and writes it into the window.
 * Liest die übersetzte Beschreibung aus den Einstellungen und schreibt diese ins Fenster.
 */
function getDescription(aGUID)
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var extPrefs = prefs.getBranch("extensions." + aGUID + ".");

    try
    {
        var extDescription = extPrefs.getComplexValue("description", Components.interfaces.nsISupportsString).data;
    } catch(e)
    {
        extDescription = extPrefs.getComplexValue("description", Components.interfaces.nsISupportsString).data;
    }

    if(extDescription.match(/^chrome\:\/\/.+/))
        extDescription = extPrefs.getComplexValue("description", Components.interfaces.nsIPrefLocalizedString).data;

    return extDescription;
}

/* Function getExtensionInfos()
 * Retrieves infos abut this extension such as the version from the extension manager.
 * Bekommt Infos wie die Version über diese Erweiterung vom Extension-Manager.
 * 
 * The code of this function is originally from zeniko's post in the MozillaZine-Forums at http://forums.mozillazine.org/viewtopic.php?p=1784670#1784670  who got it from the Mass Installer-Extension.
 */
function getExtensionInfos()
{
    const GUID = "{8637f3b0-f6d6-4496-b464-18281f83207d}";

    var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
    var extensionInfos;
    if(em.getItemForID)
        extensionInfos = em.getItemForID(GUID);
    else // getItemForID isn't available on the Aviary 1.0 branch
        extensionInfos = em.getItemList(GUID, null, {})[0];

    var infos = { description : getDescription(GUID) };
    for(var i in extensionInfos)
        infos[i] = extensionInfos[i];

    return infos;
}

/* Function resize()
 * Resizes the window so that the content fits into it and resizes the font size of the 'SEO' in the background to closely match the window's size.
 * Ändert die Fensstergröße so, dass der Inhalt reinpasst und ändert die Schriftgröße des 'SEO' im Hintergrund so, dass der text annähernd Fenstergröße hat.
 */
function resize(aTextOnly)
{
    if(!aTextOnly)
        sizeToContent();

    var width = document.defaultView.getComputedStyle(document.documentElement, "").getPropertyValue("width");
    width = parseInt(width.substring(0, width.indexOf("px"))) / 3;
    document.getElementById('seo').style.fontSize = width.toString() + 'px';

    if(!aTextOnly)
        sizeToContent();
}

function hideButton()
{
    if(top != window)
    {
        if(top.document.getElementsByTagName("browser").length > 0)
        {
            var loc = top.document.getElementsByTagName("browser")[0].getAttribute("src");
            var search = loc.substring(loc.indexOf("?") + 1, loc.length);
            if(search.indexOf("nobutton=1") > -1 || search.indexOf("nobutton=true") > -1)
            {
                document.documentElement.className = "page";
                return true;
            }
        }
    }
    return false;
}
function init(aEvent)
{
    hideButton();

    var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                    .getService(Components.interfaces.nsILocaleService)
                    .getApplicationLocale();
    var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService)
                  .createBundle("chrome://" + location.host + "/locale/about.properties", appLocale);
    var extInfos = getExtensionInfos();
    document.getElementById("description").appendChild(document.createTextNode(extInfos.description));
    document.getElementById("version").setAttribute("value", strings.GetStringFromName("version") + " " + extInfos.version);
    document.getElementById("ext-name").setAttribute("label", extInfos.name);
    document.title = strings.GetStringFromName("about") + " " + extInfos.name;

    if(top != window)
    {
        parent.document.getElementsByTagName("tabs")[0].lastChild.setAttribute("label", document.title);
    }

    resize(false);
    window.setResizable(true);
    top.window.removeEventListener("load", init, false);
}

top.window.addEventListener("load", init, false);
window.addEventListener("resize", function (aEvent) { resize(true); }, false);
