/* ***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Initial Developer of the Original Code is
Malte Kraus.

Contributor(s):

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

function seoDragDrop(aPopup, aSearchbar)
{
    this.mPopup = aPopup;
    this.mSearchbar = aSearchbar;

    this.observer = new function()
    {
        function _getLinkFromSearchPlugin(aFileURL)
        {
            var fileProtocolHandler = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                                .getService(Components.interfaces.nsIFileProtocolHandler);
            var file = fileProtocolHandler.getFileFromURLSpec(aFileURL);

            // open an input stream from file
            var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                                    .createInstance(Components.interfaces.nsIFileInputStream);
            istream.init(file, 0x01, 0444, 0);
            istream.QueryInterface(Components.interfaces.nsILineInputStream);

            // read lines into array
            var line = {}, lines = [], hasmore;
            do
            {
                hasmore = istream.readLine(line);
                if(line.value.indexOf("#") > - 1) // make sure that nothing commented out is found...
                    line.value = line.value.substring(line.value.indexOf("#"), line.value.length); // ...by removing all comments

                if(!/^\s*$/.test(line.value)) // if the string doesn't only contain whitespace
                    lines.push(line.value);
           } while(hasmore);

            istream.close();

            // try to find the host
            var searchForm = false;
            var sForm = "searchForm=\"";
            for(var i = 0; i < lines.length; i++)
            {
                if(lines[i].indexOf(sForm) > - 1)
                {
                    searchForm = lines[i].substring(lines[i].indexOf(sForm) + sForm.length, lines[i].indexOf("\"", lines[i].indexOf(sForm) + sForm.length));
                    break;
                }
            }

            if(searchForm == false)
            {
                for(i = 0; i < lines.length; i++)
                {
                    if(lines[i].indexOf("action=\"") > - 1)
                    {
                        searchForm = lines[i].substring(lines[i].indexOf("action=\"") + "action=\"".length, lines[i].indexOf("\"", lines[i].indexOf("action=\"") + "action=\"".length));
                        searchForm = searchForm.replace(/^([^:\/]+:\/\/[^\/]+)\/?.+$/, "$1/"); // just protocol, the hostname, the port (if there is one) and a trailing-slash (if there is one) so http://127.0.0.1:8080/folder/page.html would turn into http://127.0.0.1:8080/
                        break;
                    }
                }
            }

            return searchForm;
        }; // getLinkFromSearchPlugin

        this.onDragStart = function(aEvent, aTransferData, aAction)
        {
            aEvent.stopPropagation();

            var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                  .getService(Components.interfaces.nsIPrefService);
            prefs = prefs.getBranch("extensions.searchengineordering.");
            var sorting = prefs.getIntPref("sort");
            if(sorting === 1 || sorting === 2)
            {
                prefs.setIntPref("sort", 0);
                document.getElementById("seo-sort-manual").click();
            }
            sorting = null;

            if(aEvent.target.className.indexOf("searchplugin-item") == -1 || aEvent.target.tagName == "menuseparator")
            {
                return false;
            }

            var pluginName = aEvent.target.getAttribute("label");
            for(var i = 1; i < aEvent.target.parentNode.childNodes.length - (this.isSearchbar ? 2 : 0); i++)
            {
                if(aEvent.target.isSameNode(aEvent.target.parentNode.childNodes[i]))
                {
                    var pluginNumber = i;
                    break;
                }
            }
            if(typeof pluginNumber == "undefined")
            {
                return false;
            }

            var pluginLocation = aEvent.target.getAttribute("id");
            pluginLocation = "file:/" + pluginLocation.substring(7, pluginLocation.length);
            pluginLocation = pluginLocation.replace(/\%5C/g, "/");
            pluginLocation = pluginLocation.replace(/^file\:\/{4}([^\/])$/, "file:///$1");

            var linkToEngine = _getLinkFromSearchPlugin(pluginLocation);

            aTransferData.data = new TransferData();
            aTransferData.data.addDataForFlavour("application/x-moz-seo-item", pluginNumber);
            if(linkToEngine != false)
            {
                var htmlLink = "<a href=\"" + linkToEngine + "\">" + pluginName + "<\/a>";
                aTransferData.data.addDataForFlavour("application/x-moz-url", linkToEngine);
                aTransferData.data.addDataForFlavour("application/xhtml+xml", htmlLink);
                aTransferData.data.addDataForFlavour("text/html", htmlLink);
                //aTransferData.data.addDataForFlavour("text/unicode", pluginName + " (" + linkToEngine + ")"); // this would cause the current tab to do a google search for the current search engines' name and URL
            }
            //else
                //aTransferData.data.addDataForFlavour("text/unicode", pluginName); // this would cause the current tab to do a google search for the current search engines' name

            return true;
        }; // onDragStart

        this.onDragOver = function(aEvent, aFlavour, aSession)
        {
            aEvent.stopPropagation();

            if(aEvent.target.tagName == "menupopup" || !aSession.isDataFlavorSupported("application/x-moz-seo-item")
               || aEvent.target.isSameNode(aSession.sourceNode))
                aSession.canDrop = false;
            else
            {
                if(aEvent.target.boxObject.screenY + (aEvent.target.boxObject.height / 2) >= aEvent.screenY
                   || (aEvent.target.className.indexOf("searchplugin-item") == -1 && this.isSearchbar))
                   // indicator should be over the menuitem
                {
                    if(aEvent.target.className.indexOf("searchplugin-item") == -1 && this.isSearchbar)
                    {
                        if(aEvent.target.previousSibling && aEvent.target.previousSibling.previousSibling)
                            aEvent.target.previousSibling.previousSibling.setAttribute("dragover-bottom", "true");
                    }
                    else
                    {
                        aEvent.target.setAttribute("dragover-top", "true");
                        if(aEvent.target.hasAttribute("dragover-bottom"))
                            aEvent.target.removeAttribute("dragover-bottom");
                    }
                }
                else // indicator should be under the menuitem
                {
                    aEvent.target.setAttribute("dragover-bottom", "true");
                    if(aEvent.target.hasAttribute("dragover-top"))
                        aEvent.target.removeAttribute("dragover-top");
                }

                aSession.canDrop = true;
            }

            return aSession.canDrop;
        }; // onDragOver

        this.onDragExit = function(aEvent, aSession)
        {
            aEvent.stopPropagation();

            if(aEvent.target.hasAttribute("dragover-top"))
                aEvent.target.removeAttribute("dragover-top");
            if(aEvent.target.hasAttribute("dragover-bottom"))
                aEvent.target.removeAttribute("dragover-bottom");

            if(this.isSearchbar && aEvent.target.previousSibling
               && aEvent.target.previousSibling.previousSibling
               && aEvent.target.previousSibling.previousSibling.hasAttribute("dragover-bottom"))
            {
                aEvent.target.previousSibling.previousSibling.removeAttribute("dragover-bottom");
            }

            return aSession.canDrop;
        }; // onDragExit

        this.onDrop = function(aEvent, aDropData, aSession)
        {
            aEvent.stopPropagation();
            this.onDragExit(aEvent, aSession); // so that all drag-indicatos go away (don't know why but sometimes they did not)

            if(aDropData.data == "" || aEvent.target.tagName == "menupopup")
                aSession.canDrop = false;

            if(aSession.canDrop)
            {
                var pluginNumber = parseInt(aDropData.data);
                if(isNaN(pluginNumber) || pluginNumber > aEvent.target.parentNode.childNodes.length - (this.isSearchbar ? 2 : 0) || pluginNumber < 1 || pluginNumber < 1)
                    aSession.canDrop = false;
                else
                {
                    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                          .getService(Components.interfaces.nsIPrefService);
                    prefs = prefs.getBranch("browser.search.order.");
                    var prefOrder = this._parent._getPrefOrder();

                    var target = aEvent.target;

                    if(target.isSameNode(target.parentNode.lastChild) && this.isSearchbar)
                         target = target.previousSibling;

                    if(target.boxObject.screenY + (target.boxObject.height / 2) >= aEvent.screenY ||
                      (target.className.indexOf("searchplugin-item") == -1 && target.isSameNode(target.parentNode.lastChild.previousSibling)))
                       // insert before the event's target
                    {

                        target.parentNode.insertBefore(target.parentNode.childNodes[pluginNumber], target); // change the order in the popup-menu
                    }
                    else // insert after the event's target
                    {
                        if(this.isSearchbar)
                            target.parentNode.insertBefore(target.parentNode.childNodes[pluginNumber], target.nextSibling);
                        else
                        {
                            target.parentNode.insertBefore(target.parentNode.childNodes[pluginNumber], target);

                            var targetNumber;
                            for(var i = 0; i < target.parentNode.childNodes.length; i++)
                            {
                                if(target.parentNode.childNodes[i].isSameNode(target))
                                {
                                    targetNumber = i;
                                    break;
                                }
                            }
                            pluginNumber = targetNumber - 1;

                            target.parentNode.insertBefore(target, target.parentNode.childNodes[pluginNumber]);
                        }
                    } // where to insert

                    var newClass, oldClass;
                    var str = Components.classes["@mozilla.org/supports-string;1"]
                                        .createInstance(Components.interfaces.nsISupportsString);
                    // save the changes:
                    for(i = 1; i < target.parentNode.childNodes.length - (this.isSearchbar ? 2 : 0); i++)
                    {
                        str.data = target.parentNode.childNodes[i].getAttribute("label");
                        prefs.setComplexValue(i, Components.interfaces.nsISupportsString, str);

                        if(target.parentNode.childNodes[i].className.indexOf("last-searchplugin-item") > -1)
                        {
                            oldClass = target.parentNode.childNodes[i].className.split(" ");
                            newClass = "";
                            for(var j = 0; j < oldClass.length; j++)
                            {
                                if(oldClass[j] != "last-searchplugin-item")
                                    newClass += " " + oldClass[j];
                            }
                            target.parentNode.childNodes[i].className = newClass.substring(1, newClass.length);
                        }
                    }
                    target.parentNode.childNodes[target.parentNode.childNodes.length - (this.isSearchbar ? 3 : 1)].className += " last-searchplugin-item";
                }
            } // aSession.canDrop

            return aSession.canDrop;
        }; // onDrop

        this.getSupportedFlavours = function()
        {
            var flavours = new FlavourSet();
            flavours.appendFlavour("application/x-moz-seo-item");
            return flavours;
        }; // getSupportedFlavours
    } (); // observer

    /* Function seo.addDragAndDrop()
     * Adds the needed event-listeners to the menupopup containing search plugins so that they can be dragged to change their order.
     * Fügt die Event-Listener zu dem menupopup hinzu, damit die darin enthaltenen Menü-Einträge mit den Search Plugins per Drag and Drop sortiert werden können.
     */
    this.addDragAndDrop = function(aObserver, aMenupopup)
    {
        // add the drag-and-drop-event-handlers:
        this.mPopup.addEventListener("draggesture",   function(aEvent) { nsDragAndDrop.startDrag(aEvent, aObserver); }, false);
        this.mPopup.addEventListener("dragover",      function(aEvent) { nsDragAndDrop.dragOver (aEvent, aObserver); }, false);
        if("onDragEnter" in aObserver)
            this.aPopup.addEventListener("dragenter", function(aEvent) { aObserver.onDragEnter  (aEvent);            }, false);
        this.mPopup.addEventListener("dragexit",      function(aEvent) { nsDragAndDrop.dragExit (aEvent, aObserver); }, false);
        this.mPopup.addEventListener("dragdrop",      function(aEvent) { nsDragAndDrop.drop     (aEvent, aObserver); }, false);

        return true;
    }; // addDragAndDrop
} // seoDragDrop
