/* ***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Initial Developer of the Original Code is
Malte Kraus.

Contributor(s):

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/
var gTranslation; // will contain the stringbundle with some localized Strings, changed by init()
var gInstantApply = false; // apply changes immediately? changed by init()

/* Function move:
 * Moves the search plugin up or down in the tree view and saves the changes.
 * Verschiebt die Suchmaschinen nach oben oder unten und speichert die Änderungen.
 */
function move(aDirection, aIndex)
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var allPrefs = prefs.getBranch("");
    if(!gInstantApply)
        enable(true);
    var tree = document.getElementById("tree");

    if(tree.getAttribute("disabled") == "true")
    {
        toggleAlphabetic(0);
    }

    if(!aIndex)
        aIndex = tree.currentIndex;

    var current, other;
    if(aDirection != "up" && aDirection != "down" && aDirection != "top" && aDirection != "bottom")
        return false;
    if((aDirection == "up" && aIndex == 0) || (aDirection == "down" && aIndex == tree.childNodes[2].childNodes.length - 1))
        return false;
    if(aDirection == "up" || aDirection == "down")
    {
        var move = (aDirection == "up") ? -1 : 1;
        current = tree.lastChild.childNodes[aIndex]; // the selected search plugin
        if(aDirection == "up") // move the search plugin up
        {
            other = current.previousSibling;

            current.firstChild.firstChild.setAttribute("label", (((aIndex < 10) ? "0" : "") + aIndex.toString())); // update the number of the selected plugin
            other.firstChild.firstChild.setAttribute("label", (((aIndex + 1 < 10) ? "0" : "") + (aIndex + 1).toString())); // update the number of the plugin before the selected one

            tree.lastChild.insertBefore(current, other); // move the selected plugin before the previous one

            if(gInstantApply)
            {
                var orderPrefs = prefs.getBranch("browser.search.order.");

                var str = Components.classes["@mozilla.org/supports-string;1"]
                                    .createInstance(Components.interfaces.nsISupportsString);
                str.data = current.firstChild.lastChild.getAttribute("label");
                orderPrefs.setComplexValue(aIndex, Components.interfaces.nsISupportsString, str);
                str.data = other.firstChild.lastChild.getAttribute("label");
                orderPrefs.setComplexValue(aIndex + 1, Components.interfaces.nsISupportsString, str);
            }
        }
        else // move the search plugin down
        {
            other = current.nextSibling;

            other.firstChild.firstChild.setAttribute("label", (((aIndex + 1 < 10) ? "0" : "") + (aIndex + 1).toString()));
            current.firstChild.firstChild.setAttribute("label", (((aIndex + 2 < 10) ? "0" : "") + (aIndex + 2).toString()));

            tree.childNodes[2].insertBefore(other, current); // move the selected search plugin down

            if(gInstantApply)
            {
                var orderPrefs = prefs.getBranch("browser.search.order.");

                var str = Components.classes["@mozilla.org/supports-string;1"]
                                    .createInstance(Components.interfaces.nsISupportsString);
                str.data = other.firstChild.lastChild.getAttribute("label");
                orderPrefs.setComplexValue(aIndex + 1, Components.interfaces.nsISupportsString, str);
                str.data = current.firstChild.lastChild.getAttribute("label");
                orderPrefs.setComplexValue(aIndex + 2, Components.interfaces.nsISupportsString, str);
            }
        }

        tree.view.selection.select(aIndex + move); // select the search plugin that was selected until it was moved
    }
    else
    {
        if(aDirection == "top") // the search plugin should be the first
        {
            current = tree.lastChild.childNodes[aIndex];
            var first = tree.lastChild.firstChild;
            tree.lastChild.insertBefore(current, first);
            tree.view.selection.select(0);
            treeNumber();
        }
        else // the search plugin should be the last
        {
            for(var i = aIndex + 1; i < tree.childNodes[2].childNodes.length; i++)
            {
                current = tree.lastChild.childNodes[aIndex];
                other = tree.lastChild.childNodes[i];
                other.firstChild.firstChild.setAttribute("label",(((aIndex + 1 < 10) ? "0" : "") + (aIndex + 1).toString()));
                current.firstChild.firstChild.setAttribute("label",(((aIndex + 2 < 10) ? "0" : "") + (aIndex + 2).toString()));
                tree.lastChild.insertBefore(other, current);
                tree.view.selection.select(i);
                aIndex = tree.currentIndex;
            }
        }

        if(gInstantApply)
            save();
    }

    var boxobject = tree.boxObject;
    boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
    boxobject.ensureRowIsVisible(tree.currentIndex);
    return true;
}

/* Function treeOrder:
 * Saves the order of the search plugins.
 * Speichert die Reihenfolge der Search Plugins.
 */
function treeOrder()
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("browser.search.order.");
    var tree = document.getElementById("tree");
    var noPref = false;
    var currentLabel = " ", currentPref = " ";
    var existent = false;
    for(var i = 0; i < tree.view.rowCount; i++)
    {
        currentLabel = tree.view.getItemAtIndex(i).firstChild.lastChild.getAttribute("label");
        try      { currentPref = prefs.getComplexValue(i + 1, Components.interfaces.nsISupportsString).data; }
        catch(e) { noPref = true;                          }
        if(prefs.getPrefType(i + 1) !== prefs.PREF_STRING || noPref === true)
        {
            save();
            break;
        }
        if(currentLabel !== currentPref)
        {
            for(var j = i; j < tree.view.rowCount; j++)
            {
                if(tree.view.getItemAtIndex(j).firstChild.lastChild.getAttribute("label") === currentPref)
                {
                    var elems = Array();
                    for(i = 0; prefs.getPrefType(i + 1) == prefs.PREF_STRING; i++)
                    {
                        try { elems[i] = prefs.getComplexValue(i + 1, Components.interfaces.nsISupportsString).data; }
                        catch(e) { break; }
                    }
                    rebuildTree(elems);
                    return true;
                }
            }

            // There is a pref which points a to non-existing search plugin
            var str = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            str.data = currentLabel;
            prefs.setComplexValue(i + 1, Components.interfaces.nsISupportsString, str);
        }
    }
    return true;
}

/* Function trreeNumber:
 * Gives every row its number.
 * Schreibt die Nummer in die erste Spalte.
 */
function treeNumber()
{
    var tree = document.getElementById("tree");
    for(var i = (typeof arguments[0] != "undefined") ? arguments[0] : 0; i < tree.view.rowCount; i++)
    {
        tree.view.getItemAtIndex(i).firstChild.firstChild.setAttribute("label",(i < 9) ? ( "0" + (i+1).toString()) : (i+1).toString());
    }
}

/* Function init:
 * Starts all the functions that have to be executed at startup.
 * Führt alle Funktionen aus, die beim starten gestartet werden müssen.
 */
function init()
{
    gTranslation = document.getElementById("translation");

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var seoPrefs = prefs.getBranch("extensions.searchengineordering.");
    prefs = prefs.getBranch("");

    //* (un)check, dis-/enable, (un)hide all checkboxes controlling prefs:
    var downloadMoreURL = seoPrefs.getComplexValue("addEngines.url", Components.interfaces.nsISupportsString).data;
    if(downloadMoreURL == "chrome://" + location.host + "/locale/defaults.properties")
        downloadMoreURL = seoPrefs.getComplexValue("addEngines.url", Components.interfaces.nsIPrefLocalizedString).data;
    document.getElementById("downloadmore").   setAttribute("href",     downloadMoreURL);
    document.getElementById("addengines-url").value                   = downloadMoreURL;
    document.getElementById("addengines-url").disabled = seoPrefs.getBoolPref("addEngines.hide");
    document.getElementById("addengines-hide").setAttribute("checked",  seoPrefs.getBoolPref("addEngines.hide"));
    document.getElementById("hidemenuitem").   setAttribute("checked", !seoPrefs.getBoolPref("options.hidemenuitem"));
    var openintab = document.getElementById("openintab");
    openintab.setAttribute("checked",  seoPrefs.getBoolPref("options.openintab"));
    openintab.setAttribute("disabled", openintab.previousSibling.getAttribute("checked") == "false");
    document.getElementById("tabbox").selectedIndex = seoPrefs.getIntPref("options.lastSelectedTab");
    document.getElementById("tabbox").firstChild.addEventListener("select",
        function(aEvent)
        {
            return saveSelectedTab(aEvent.currentTarget);
        }, false);

    // check if the prefs should be applied immediately: (Deer Park-only)
    try
    {
        if(prefs.getPrefType("browser.preferences.instantApply") == prefs.PREF_BOOL && prefs.getBoolPref("browser.preferences.instantApply"))
            gInstantApply = true;
        else
            gInstantApply = false;
    } catch(e)
    {
        gInstantApply = false;
    }

    document.getElementById("apply").setAttribute("hidden",  gInstantApply.toString());
    document.getElementById("cancel").setAttribute("hidden", gInstantApply.toString());

    removeDeletedEngines();
    sortPlugins();

    if(Math.floor(window.innerHeight / 10) == 40 && Math.floor(window.innerWidth / 10)== 40)
    {
        sizeToContent();
        resizeTo(innerWidth * 2, innerHeight * 2);
    }
    window.setResizable(true);

    enable(false);

    if(!gInstantApply)
        window.addEventListener("unload", askToSave, true);
    window.addEventListener("unload", prefObserver.unregister, true);
}

/* Function tooglePrefs:
 * Sets a given preference to true if it's value is false and to false if it's value is true.
 * Verändert den Wert einer übergebenen Einstellung von true auf false oder andersherum.
 */
function togglePref(aPrefName, aPrefValue)
{
    if(gInstantApply)
    {
        try
        {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
            prefs = prefs.getBranch("extensions.searchengineordering.");
            switch(typeof aPrefValue)
            {
                case "string":
                {
                    var str = Components.classes["@mozilla.org/supports-string;1"]
                                        .createInstance(Components.interfaces.nsISupportsString);
                    str.data = aPrefValue;
                    prefs.setComplexValue(aPrefName, Components.interfaces.nsISupportsString, str);
                    break;
                }
                case "number":
                {
                    prefs.setIntPref(aPrefName, aPrefValue);
                    break;
                }
                case "boolean":
                {
                     prefs.setBoolPref(aPrefName, aPrefValue);
                     break;
                }
                case "undefined":
                case null:
                default:
                {
                    prefs.setBoolPref(aPrefName, !prefs.getBoolPref(aPrefName));
                }
            }
            return true;
        } catch(e) { }
    }
    else
    {
        enable(true);
        return true;
    }

    return false;
}

/* Function openURL:
 * Opens a URL
 * Öffnet eine URL.
 */
function openURL(aURL, aTarget)
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.searchengineordering.");
    var tab = prefs.getBoolPref("options.openintab");
    if(tab)
    {
        try
        {
            opener.gBrowser.delayedOpenTab(aURL);
            return true;
        }
        catch(e)
        {
            var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
            var recentWindow = wm.getMostRecentWindow("navigator:browser");
            if(recentWindow)
            {
                // Use an existing browser window
                recentWindow.delayedOpenTab(aURL);
                return true;
            }
            else
            {
                window.open(aURL, aTarget, "");
            }
        }
    }
    window.open(aURL, aTarget, "");
    return true;
}

/* Function resetOrder:
 * Iterates through the prefs and resets them.
 * Geht alle Einstellungen durch und setzt sie zurück.
 */
function resetOrder()
{
    if(!gInstantApply)
        enable(false);

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var seoPrefs = prefs.getBranch("extensions.searchengineordering.");

    if(seoPrefs.getBoolPref("reset.ask") == true && (typeof arguments[0] == "undefined" || arguments[0] != 'silent'))
    {

        var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                        .getService(Components.interfaces.nsILocaleService)
                        .getApplicationLocale();
        var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                      .getService(Components.interfaces.nsIStringBundleService)
                      .createBundle("chrome://" + location.host + "/locale/global.properties", appLocale);

        if(!strings)
            return false;

        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                      .getService(Components.interfaces.nsIPromptService);
        var askAgain = {value:false};
        var reset = prompts.confirmEx(window,
                                      strings.GetStringFromName("name"),
                                      gTranslation.getString("resetQuestion"),
                                      (1 * 3) + (256 * 4), "3", "4", "",
                                      strings.GetStringFromName("dontAskAgain"), askAgain);

        if(askAgain.value && reset == 0)
            seoPrefs.setBoolPref("reset.ask",false);
        if(reset != 0)
            return false;
    }

    var orderPrefs = prefs.getBranch("browser.search.order.");
    var prefBranchInternal = orderPrefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    for(var i = 1; orderPrefs.getPrefType(i.toString()) == orderPrefs.PREF_STRING; i++)
    {
        try
        {
            prefBranchInternal.clearUserPref(i.toString());
        } catch(e) {}
    }
    return true;
}

/* Function deleteEngine:
 * Deletes the cuurently selected search plugin.
 * Löscht das gerade markierete Search Plugin.
 */
function deleteEngine()
{
    if(document.getElementById("tree").view.selection.count != 1)
        return -1;
    var selected = document.getElementById("tree").currentIndex;

    var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                    .getService(Components.interfaces.nsILocaleService)
                    .getApplicationLocale();
    var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService)
                  .createBundle("chrome://" + location.host + "/locale/global.properties", appLocale);

    if(!strings)
        return -2;

    var item = document.getElementById("tree").view.getItemAtIndex(selected);
    var itemName = item.firstChild.lastChild.getAttribute("label");
    var itemLocations = new Array();
    itemLocations[0] = item.getAttribute("id");
    itemLocations[0] = "file:/" + itemLocations[0].substring(7, itemLocations[0].length);
    itemLocations[0] = itemLocations[0].replace(/\%5C/g, "/");
    itemLocations[0] = itemLocations[0].replace(/^file\:\/{4}([^\/])/, "file:///$1");
    itemLocations[1] = item.firstChild.childNodes[1].getAttribute("src");

    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.searchengineordering.");

    var remove = true;
    if(prefs.getBoolPref("remove.ask"))
    {
        var askAgain = {value:false};
        remove = prompts.confirmEx(window,
                                   strings.GetStringFromName("name"),
                                   strings.GetStringFromName("deleteConfirm1") + itemName + strings.GetStringFromName("deleteConfirm2"),
                                   (1 * 3) + (256 * 4), "3", "4", "",
                                   strings.GetStringFromName("dontAskAgain"), askAgain) == 0;
        if(askAgain.value == true && remove)
            prefs.setBoolPref("remove.ask", false);
    }

    if(remove)
    {
        var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                 .createInstance(Components.interfaces.nsIFileProtocolHandler);
        var searchPlugin = url2file.getFileFromURLSpec(itemLocations[0]);
        if(itemLocations[1] != "")
            var image = url2file.getFileFromURLSpec(itemLocations[1]);
        else
            var image = false;

        if(!searchPlugin.exists() || (image !== false && !image.exists()))
            prompts.alert(window, strings.GetStringFromName("name"), strings.GetStringFromName("deleteNotExisting"))

        if((searchPlugin.exists() && !searchPlugin.isWritable()) && (image !== false && image.exists() && !image.isWritable()))
        {
            prompts.alert(window, strings.GetStringFromName("name"),
                          strings.GetStringFromName("deleteNoRightsBegin")
                           + searchPlugin.path
                           + strings.GetStringFromName("deleteNoRightsAnd")
                           + image.path
                           + strings.GetStringFromName("deleteNoRightsEnd"));
            return -3;
        }
        else
        {
            if(searchPlugin.exists() && !searchPlugin.isWritable())
            {
                prompts.alert(window, strings.GetStringFromName("name"),
                              strings.GetStringFromName("deleteNoRightsBegin")
                               + searchPlugin.path
                               + strings.GetStringFromName("deleteNoRightsEnd"));

                image.remove(false);

                return -0.5;
            }
            if(image !== false && image.exists() && !image.isWritable())
            {
                prompts.alert(window, strings.GetStringFromName("name"),
                              strings.GetStringFromName("deleteNoRightsBegin")
                               + image.path
                               + strings.GetStringFromName("deleteNoRightsEnd"));

                searchPlugin.remove(false);

                return 0.5;
            }
        }

        if(searchPlugin.exists() && searchPlugin.isWritable())
        {
            searchPlugin.remove(false);
        }
        if(image !== false && image.exists() && image.isWritable())
        {
            image.remove(false);
        }
        item.parentNode.removeChild(item);

        if(!gInstantApply)
            enable(true);

        treeNumber();
        save();

        return 1;
    }

    return -4;
}

/* Function alphabetic:
 * Sorts the search plugins alphabetically.
 * Sortiert die Suchmaschinen alphabetisch.
 */
function alphabetic(aDirection)
{
    if(aDirection !== 1 && aDirection !== 2)
        return false;
    
    var tree = document.getElementById('tree');
    var deep = 0;
    var compared;

    /* Function compare:
     * Determines, whether the nth (n is -deep) character of the first argument is before or after in the alphabet than the nth character in the second argument. (is needed for alphabetic())
     * Findet heraus, ob der nte (n ist -deep) Buchstabe des ersten Parameters im Alphabet vor oder nach dem nten Buchstaben des zweiten Parameters vorkommt. (wird für alphabetic() gebraucht)
     */
    function compare(aFirst, aSecond, aDeep, aDirection)
    {
        if(aDirection !== 1 && aDirection !== 2)
            aDirection = 1;

        aFirst =  aFirst.toLowerCase();
        aSecond = aSecond.toLowerCase();

        if(aFirst === aSecond) // both search plugins have the same name ==> do nothing
            return 0;
        else if(aFirst.charCodeAt(-aDeep) > aSecond.charCodeAt(-aDeep) || -aDeep > aSecond.length) // the upper plugin should go down
            return aDirection === 1 ? 1 : 0;
        else if(aFirst.charCodeAt(-aDeep) < aSecond.charCodeAt(-aDeep) || -aDeep > aFirst.length) // both plugins are in the right order
            return aDirection === 1 ? 0 : 1;

        return aDeep-1; // the nth character is the same ==> check the n+1th charcter
    }

    for(var i = 0; i < tree.view.rowCount - 1; i++)
    {
        compared = compare(tree.view.getItemAtIndex(i).firstChild.lastChild.getAttribute("label"), tree.view.getItemAtIndex(i + 1).firstChild.lastChild.getAttribute("label"), deep, aDirection);
        if(compared < 0) // compared is negative ==> both characters were the same
        {
            deep = compared;
            i = i - 1; // continue the loop with the same value of i but with another value of compared again
        }
        if(compared >= 0) // compared is not negative ==> both characters were different ==> we know the order of these two search plugins
            deep = 0; // the nth character should be the first character when the next two search plugins will be compared
        if(compared === 1) // the second should be before the first search plugin
        {
            tree.view.getItemAtIndex(i).parentNode.insertBefore(tree.view.getItemAtIndex(i).nextSibling, tree.view.getItemAtIndex(i));
            if(i !== 0) // we have to check if the (new) upper one should be before the search plugin before itself
                i  = i - 2;
            else // but this isn't possible if the search plugin is at the top
                i  = i - 1;
        }
    }
    treeNumber(); // correct the row with the Number:
    if(!gInstantApply)
        enable(true);
    return true;
}

/* Function save:
 * Saves the search engines' order
 * Speichert die Reihenfolge der Suchmaschinen.
 */
function save()
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var orderPrefs = prefs.getBranch("browser.search.order.");
    var seoPrefs = prefs.getBranch("extensions.searchengineordering.");
    prefs = prefs.getBranch(null);


    resetOrder("silent"); // reset the old order (there would be a pref too much when an engine has been removed)
    var tree = document.getElementById("tree");
    var str = Components.classes["@mozilla.org/supports-string;1"]
                        .createInstance(Components.interfaces.nsISupportsString);
    if(tree.view) // on unload not always available
    {
        for(var i = 0; i < tree.view.rowCount; i++)
        {
            str.data = tree.view.getItemAtIndex(i).firstChild.lastChild.getAttribute("label");
            orderPrefs.setComplexValue(i + 1, Components.interfaces.nsISupportsString, str);
        }
    }
    else
    {
        for(var i = 0; i < tree.lastChild.childNodes.length; i++)
        {
            str.data = tree.lastChild.childNodes[i].firstChild.lastChild.getAttribute("label");
            orderPrefs.setComplexValue(i + 1, Components.interfaces.nsISupportsString, str);
        }
    }

    seoPrefs.setIntPref("sort", parseInt(tree.getAttribute("sortDirection")));
    str.data = document.getElementById("addengines-url").value;
    seoPrefs.setComplexValue("addEngines.url", Components.interfaces.nsISupportsString, str);

    enable(false);

    var checkboxes = document.getElementsByTagName("checkbox");
    for(i = 0; i < checkboxes.length; i++)
    {
        var pref = false;
        if(checkboxes[i].hasAttribute("pref"))
            pref = checkboxes[i].getAttribute("pref");
        else if(checkboxes[i].hasAttribute("command"))
        {
            var cmd = document.getElementById(checkboxes[i].getAttribute("command"));
            if(cmd && cmd.hasAttribute("pref"))
                pref = cmd.getAttribute("pref");
        }
        if(pref !== false)
        {
            if(pref.charAt(0) == "!")
                seoPrefs.setBoolPref(pref.substring(1, pref.length), !checkboxes[i].checked);
            else
                seoPrefs.setBoolPref(pref, checkboxes[i].checked);
        }
    }

    return true;
}

/* Function enable:
 * Enables or disables the 'Apply'-Button. (depending on the parameter bool) And saves the changes if gInstantApply is true.
 * Aktiviert oder deaktiviert den 'Übernehmen'-Button. (abhängig vom Parameter bool) Und speichert die Veränderung wenn gInstantApply true ist.
 */
function enable(bool)
{
    document.getElementById('apply').setAttribute('disabled', (!bool).toString());
}

/* Function checkCommands:
 * Checks the commands that apply on the search engines (such as up or down) if they have to be enabled or disabled.
 * Überprüft, ob die die Suchmaschinen beteffenden Aktionen (Hoch, Runter...) ein- oder ausgeschaltet werden müssen.
 */
function checkCommands()
{
    if(document.getElementById('tree').view.selection.count != 1)
    {
        document.getElementById('cmd-move-up').setAttribute    ("disabled", 'true');
        document.getElementById('cmd-move-down').setAttribute  ('disabled', 'true');
        document.getElementById('cmd-move-top').setAttribute   ('disabled', 'true');
        document.getElementById('cmd-move-bottom').setAttribute('disabled', 'true');
    }
    else
    {
        if(document.getElementById('tree').currentIndex != 0)
        {
            document.getElementById('cmd-move-up').setAttribute ("disabled", 'false');
            document.getElementById('cmd-move-top').setAttribute('disabled', 'false');
        }
        else
        {
            document.getElementById('cmd-move-up').setAttribute ("disabled", 'true');
            document.getElementById('cmd-move-top').setAttribute('disabled', 'true');
        }

        if(document.getElementById('tree').currentIndex != document.getElementById('tree').view.rowCount - 1)
        {
            document.getElementById('cmd-move-down').setAttribute  ('disabled', 'false');
            document.getElementById('cmd-move-bottom').setAttribute('disabled', 'false');
        }
        else
        {
            document.getElementById('cmd-move-down').setAttribute  ("disabled", 'true');
            document.getElementById('cmd-move-bottom').setAttribute('disabled', 'true');
        }
    }
}

/* Function rebuildTree:
 * Sorts the tree in the order, which is specified in the parameter (an array) elems.
 * Sortiert den tree in der Reihenfolge, die im Parameter elems (ein Array) angegeben wird.
 */
function rebuildTree(elems)
{
    var tree = document.getElementById('tree');
    var treeLabels = Array();
    for(var i = 0; i < tree.view.rowCount; i++)
    {
        treeLabels[i] = tree.view.getItemAtIndex(i).firstChild.lastChild.getAttribute('label');
    }
    if(elems.length != treeLabels.length)
        return treeLabels;
    for(i = 0; i < elems.length; i++)
    {
        if(elems[i] != treeLabels[i])
        {
            for(var j = 0; j < elems.length; j++)
            {
                if(elems[i] == treeLabels[j])
                {
                    if(j < i) for(j = j; j < i; j++)
                    {
                        tree.view.selection.select(j);
                        move('down');
                        treeLabels[j] = treeLabels[j+1];
                        treeLabels[j+1] = elems[i];
                    }
                    else for(j = j; j > i; j--)
                    {
                        tree.view.selection.select(j);
                        move('up');
                        treeLabels[j] = treeLabels[j-1];
                        treeLabels[j-1] = elems[i];
                    }
                    break;
                } // elems[i] == treeLabels[j]
            } // for j
        } // elems[i] != treeLabels[i]
    } // for i

    tree.view.selection.select(0);
    tree.view.selection.clearSelection();

    var boxobject = tree.boxObject;
    boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
    boxobject.ensureRowIsVisible(0);

    if(!gInstantApply)
        enable(false);
    return elems;
} // rebuildTree(elems)

/* Function askToSave:
 * Asks the user to save the changes he made when the window/tab is closed on another way than using one of the three buttons on the bottom of the window.
 * fragt den User, ob die Änderungen gespeichert werden sollen, wenn er SEO nicht über einen der Buttons verlässt.
 */
function askToSave()
{
    if(document.getElementById("apply").getAttribute("disabled") == "false")
    {
        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        var saveChanges = !prompts.confirmEx(window, document.documentElement.getAttribute("title"),
                                             gTranslation.getString("saveDiscard"),
                                             (1 * 5) + (256 * 6), "save", "dontsave", "",
                                             null, {value:false});
        if(saveChanges)
            save();

        return true;
    }
}

/* Function saveSelectedTab:
 * Saves the currently selected tab.
 * Speichert, welcher Tab gerade ausgewählt ist.
 */
function saveSelectedTab(tabbox)
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.searchengineordering.options.");
    prefs.setIntPref("lastSelectedTab", tabbox.selectedIndex);
}

/* Function toggleAlphabetic:
 * Toggles the display of the tree, asks if the user really wants to sort alphabetic, saves that change.
 * Schaltet die Anzeige des Trees um, fragt den Benutzer, ob er wirklich alphabetisch sortieren will und speichert diese Änderungen.
 */
function toggleAlphabetic(aToggleSort)
{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.searchengineordering.");
    var sort = prefs.getIntPref("sort");

    var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                    .getService(Components.interfaces.nsILocaleService)
                    .getApplicationLocale();
    var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService)
                  .createBundle("chrome://" + location.host + "/locale/global.properties", appLocale);
    if(!strings)
        return;

    if(typeof aToggleSort != "undefined" && aToggleSort !== null)
    {
        sort =  aToggleSort;
        togglePref("sort", sort);
    }

    if(sort !== 0 && sort !== 1 && sort !== 2)
    {
        prefs.setIntPref("sort", 0);
        sort = 0;
    }

    document.getElementById("tree").setAttribute("disabled", (sort !== 0).toString());
    document.getElementById("tree").setAttribute("sortDirection", sort.toString());

    if(sort === 1 || sort === 2)
        alphabetic(sort);
    checkCommands();
    checkSort(sort);
}

/* Function checkSort
 * Hides or shows the context-menu of the engines in the tree.
 * Blendet das Kontext-Menü der Suchmaschinen im Tree an oder aus.
 */
function checkSort(aSort)
{
    if(typeof aSort == "undefined" || aSort == null)
        aSort = Components.classes["@mozilla.org/preferences-service;1"].
                              getService(Components.interfaces.nsIPrefService).
                              getBranch("extensions.searchengineordering.").
                              getIntPref("sort");

    document.getElementById("context-order-manual").setAttribute("checked", (aSort !== 1 && aSort !== 2).toString());
    document.getElementById("context-order-alphabetic-az").setAttribute("checked", (aSort === 1).toString());
    document.getElementById("context-order-alphabetic-za").setAttribute("checked", (aSort === 2).toString());
}

/* Function sortPlugins:
 * Sorts the search engines alphabetically or as stated in the prefs.
 * Sortiert die Suchmaschinen alphabetisch oder wie abgespeichert.
 */
function sortPlugins()
{
    toggleAlphabetic(null);

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.searchengineordering.");
    if(prefs.getIntPref("sort") !== 1 && prefs.getIntPref("sort") !== 2)
        treeOrder();

    treeNumber(); // add the numbers-column
}

/* Function removeDeletedEngines:
 * Removes the engines that don't exist.
 * Enfernt die Suchmaschinen, die nicht vorhanden sind.
 */
function removeDeletedEngines()
{
    var tree = document.getElementById("tree");
    var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"].createInstance(Components.interfaces.nsIFileProtocolHandler);
    var img, imgPath, plugin, engine = false;
    for(var i = 0; i < tree.view.rowCount; i++)
    {
        imgPath = tree.view.getItemAtIndex(i).firstChild.childNodes[1].getAttribute("src");
        if(imgPath != "")
            img = url2file.getFileFromURLSpec(imgPath);
        else
            img = false;

        plugin = tree.view.getItemAtIndex(i).getAttribute("id");
        plugin = "file:/" + plugin.substring(7, plugin.length);
        plugin = plugin.replace(/\%5C/g, "/");
        plugin = plugin.replace(/^file\:\/{4}([^\/])/, "file:///$1");
        plugin = url2file.getFileFromURLSpec(plugin);

        if((img !== false && !img.exists()) || !plugin.exists())
        {
            var engine = document.getElementById("engine://" + encodeURIComponent(plugin.path));
            engine.parentNode.removeChild(engine);
        }
    }

    if(engine !== false)
        save();
} //removeDeletedEngines

var prefObserver = new (function ()
{
    this.register = function()
    {
        var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                    .getService(Components.interfaces.nsIPrefService);
        this._branch = prefService.getBranch("extensions.searchengineordering.");
        this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
        this._branch.addObserver("", this, false);
    };

    this.unregister = function()
    {
        if(!this._branch) return;
            this._branch.removeObserver("", this);
        this._branch = false;
    };

    this.observe =  function(aSubject, aTopic, aData)
    {
        // aSubject is the nsIPrefBranch we're observing
        // aData is the name of the pref that's been changed (relative to aSubject)
        if(aTopic != "nsPref:changed") return;
        if(typeof Components == "undefined") return;
        var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                    .getService(Components.interfaces.nsIPrefService);
        var branch = prefService.getBranch("extensions.searchengineordering.");
        switch(branch.getPrefType(aData))
        {
            case branch.PREF_STRING:
                var newValue = branch.getComplexValue(aData, Components.interfaces.nsISupportsString).data;
                break;
            case branch.PREF_BOOL:
                var newValue = branch.getBoolPref(aData);
                break;
            case branch.PREF_INT:
                var newValue = branch.getIntPref(aData);
                break;
        }
        switch (aData)
        {
            case "addEngines.hide":
            {
                document.getElementById("addengines-hide").setAttribute("checked", newValue.toString());
                break;
            }
            case "addEngines.url":
            {
                if(newValue == "chrome://" + location.host + "/locale/defaults.properties")
                    newValue = branch.getComplexValue(aData, Components.interfaces.nsIPrefLocalizedString).data;
                document.getElementById("downloadmore").setAttribute("href", newValue);
                break;
            }
            case "options.hidemenuitem":
            {
                document.getElementById("hidemenuitem").setAttribute("checked", (!newValue).toString());
                document.getElementById("openintab").setAttribute("disabled", newValue.toString());
                break;
            }
            case "options.openintab":
            {
                document.getElementById("openintab").setAttribute("checked", newValue.toString());
                break;
            }
            case "sort":
            {
                toggleAlphabetic(newValue);
                break;
            }
        }
    };
})();
prefObserver.register();