/* ***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Initial Developer of the Original Code is
Malte Kraus.

Contributor(s):

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
***** END LICENSE BLOCK ***** */
var seoObj = function(aPopup, aSearchbar)
{
    /* function init()
     * Does everything that should be done in a new window.
     * Macht alles, was beim Start eines neuen Fensters genau einaml gemacht werden sollte.
     */
    this.init = function()
    {
        try
        {
            if(typeof aSearchbar != "undefined")
                var searchbar = aSearchbar;
            else
                var searchbar = document.getElementById("searchbar");

            if(!searchbar)
                throw "the searchbar is hidden";

            if(typeof aPopup != "undefined")
                var menupopup = aPopup;
            else
                var menupopup = document.getAnonymousElementByAttribute(searchbar, "anonid", "searchbar-popup");

            if(!menupopup)
                throw "the searchbar is hidden";

            this.mSearchbar = searchbar;
            this.mPopup = menupopup;

            this.isSearchbar = searchbar.id == "searchbar";
            this.isContext = searchbar.id == "context-searchmenu";
        }
        catch(e)
        {
            return;
        }

        this.dragDrop = new seoDragDrop(menupopup, searchbar);
        this.dragDrop.isSearchbar = this.isSearchbar;
        this.dragDrop.isContext = this.isContext;
        this.dragDrop.observer.isSearchbar = this.isSearchbar;
        this.dragDrop.observer.isContext = this.isContext;
        this.dragDrop._parent = this;
        this.dragDrop.observer._parent = this;
        this.dragDrop.addDragAndDrop(this.dragDrop.observer, menupopup);  // add drag and drop functionality to the searchbar

        this.checkChangedOrder();
        this.newPluginsObserver.seo = this;
        this.newPluginsObserver.registerObserver();

        if(this == seo)
        {
            menupopup.addEventListener("popupshowing", function(aEvent) { seo.onPopupShowing(aEvent); }, false);
            menupopup.addEventListener("popuphidden",  function(aEvent) { seo.onPopupHidden(aEvent);  }, false);
            window.removeEventListener("load",         function(aEvent) { seo.init();                 }, false);

            var searchTextbox = document.getAnonymousNodes(searchbar)[1];
            this.__orig_onEnginePopupCommand = searchTextbox.onEnginePopupCommand; // make a backup of the original function
            searchTextbox.onEnginePopupCommand = this.onEnginePopupCommand; // override the function with our advanced
        }
        else if(this == seoContextSearch)
        {
            menupopup.addEventListener("popupshowing", function(aEvent) { seoContextSearch.onPopupShowing(aEvent); }, false);
            menupopup.addEventListener("popuphidden",  function(aEvent) { seoContextSearch.onPopupHidden(aEvent);  }, false);
        }

        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.searchengineordering.options.");
        if(prefs.getBoolPref("hidemenuitem"))
        {
            var menuitem = document.getElementById("seo-open");
            if(menuitem)
                menuitem.parentNode.removeChild(menuitem);
        }

        // add this search to the searchbar:
        var contextMenu = document.getElementById("contentAreaContextMenu");
        contextMenu.addEventListener("popupshowing", this.onContextMenuShowing, false);

        // Compatibility with ContextSearch extension by Cusser:
        if(typeof contextsearch != "undefined" && typeof window.seoContextSearch == "undefined")
        {
            var contextPopup = document.getElementById("context-searchmenu-popup");
            var contextSearchMenu = document.getElementById("context-searchmenu");
            window.seoContextSearch = new seoObj(contextPopup, contextSearchMenu);
            seoContextSearch.init();
        }

        this.sortPlugins(this._getPrefOrder());
        this.alphabetical();
    }; // init

    /* function sortPlugins(
     * Sorts the search plugins in the search bar without the need to restart Fx. (also sorts the search plugins in the profile folder (Deer Park only) what Fx can't do itself by now (20050802))
     * Sortiert die Suchmaschinen in der Suchleiste, ohne Firefox neu starten zu müssen. (sortiert auch die Suchmaschinen im Profilordner was Fx derzeit (2.8.2005) nicht kann)
     */
    this.sortPlugins = function(aSorted)
    {
        if(!this.mSearchbar || !this.mPopup)
            return false;

        var menu = this.mPopup;

        for(var i = 0; i + 1 < aSorted.length; i++)
        {
            if(menu.getElementsByAttribute('label', aSorted[i + 1]).length >= 1) // perhaps there is no menuitem with this label (e.g. because of old prefs)
            {
                menu.insertBefore(menu.getElementsByAttribute('label', aSorted[i + 1])[0], menu.childNodes[i]);

                if(menu.childNodes[i] && menu.childNodes[i].className.indexOf("last-searchplugin-item") > -1)
                    menu.childNodes[i].className = menu.childNodes[i].className.replace(/ last-searchplugin-item/, "");
            }
        }

        if(menu.firstChild.tagName != "template")
            menu.insertBefore(menu.getElementsByTagName("template")[0], menu.firstChild); // so that the index in the menu is the same as in the prefs (the prefs begin counting with 1 whereas arrays begin with 0)

        var menuseparator = menu.getElementsByTagName("menuseparator");
        if(menuseparator.length > 0 && !menuseparator[0].isSameNode(menu.lastChild.previousSibling))
            menu.insertBefore(menuseparator[0], menu.lastChild);

        var addEngines = menu.getElementsByAttribute("anonid", "addengine-menuitem");
        if(addEngines.length > 0 && !addEngines[0].isSameNode(menu.lastChild))
        {
            menu.insertBefore(addEngines[0], menu.lastChild);
            menu.insertBefore(menu.lastChild, addEngines[0].previousSibling);
        }

        if(this.isSearchbar && addEngines[0])
            if(menu.lastChild.previousSibling.previousSibling.className.indexOf("last-searchplugin-item") == -1)
                menu.lastChild.previousSibling.previousSibling.className += " last-searchplugin-item";

        else if(this.isContext)
            if(menu.lastChild.className.indexOf("last-searchplugin-item") == -1)
                menu.lastChild.className += " last-searchplugin-item";

        return true;
    }; // sortPlugins

    /* function checkChangedOrder()
     * Checks if the order of the search plugins in the search bar is different than the order in the prefs. Starts sortPlugins if they are different.
     * Überprüft, ob die Suchmaschinen in der Suchleiste gleich der Reihenfolge in den Einstellungen ist. Ruft sortPlugins auf, wenn nicht.
     */
    this.checkChangedOrder = function()
    {
        if(!this.mSearchbar || !this.mPopup)
            return false;

        var items = this.mPopup;

        var menuOrder = new Array();
        for(var i = 1; i < items.length; i++)
        {
            menuOrder.push(items[i].getAttribute('label'));
        }

        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("browser.search.order.");
        var prefOrder = this._getPrefOrder();

        if(menuOrder != prefOrder)
            return this.sortPlugins(prefOrder);

        return true;
    }; // checkChangedOrder

    /* Function checkForNewEngines:
     * Gives every search plugin the class 'searchplugin-item'.
     * Gibt jeder Suchmaschine die Klasse 'searchplugin-item'.
     */
    this.checkForNewEngines = function(aEvent)
    {
        var searchPopup = aEvent.target, newEngines = false;
        for(var i = 1; i < searchPopup.childNodes.length - (this.isSearchbar ? 2 : 0); i++)
        {
            // give every menuitem a class so that they can be styled (needed for the drag & and drop indicator)
            if(searchPopup.childNodes[i].className.indexOf("searchplugin-item") == -1)
            {
                searchPopup.childNodes[i].className += " searchplugin-item";

                if(searchPopup.childNodes[i].className == " searchplugin-item")
                    searchPopup.childNodes[i].className = "searchplugin-item";

                if(this.isSearchbar)
                {
                    searchPopup.insertBefore(searchPopup.childNodes[i], searchPopup.lastChild.previousSibling);
                }
                else
                {
                    searchPopup.insertBefore(searchPopup.childNodes[i], searchPopup.lastChild);
                    searchPopup.insertBefore(searchPopup.lastChild, searchPopup.lastChild.previousSibling);
                }

                newEngines = true;

                i = i - 1;
            }
        }

        if(newEngines)
        {
            this.alphabetical();
            this.sortPlugins(this._getPrefOrder());
        }

        if(searchPopup.childNodes[--i].className.indexOf("last-searchplugin-item") == -1)
            searchPopup.childNodes[i].className += " last-searchplugin-item";
    }; // checkForNewEngines

    /* Function checkForRemovedEngines:
     * Hides any engines that don't exist anymore.
     * Versteckt alle Suchmaschinen, die nicht mehr vorhanden sind.
     */
    this.checkForRemovedEngines = function(aEvent)
    {
        var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                 .createInstance(Components.interfaces.nsIFileProtocolHandler);
        var img, plugin, shouldSave = false, searchPopup = aEvent.target;
        for(var i = 1; i < searchPopup.childNodes.length - (this.isSearchbar ? 2 : 0); i++)
        {
            plugin = searchPopup.childNodes[i].getAttribute("id").replace(/^engine:/, "file:/");
            plugin = plugin.replace(/\%5C/g, "/");
            plugin = url2file.getFileFromURLSpec(plugin);

            if(!plugin.exists())
            {
                searchPopup.removeChild(searchPopup.childNodes[i]);
                if(shouldSave === false)
                    shouldSave = i;
            }
        }

        if(shouldSave !== false)
        {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                  .getService(Components.interfaces.nsIPrefService);
            prefs = prefs.getBranch("browser.search.order.");
            var str = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            for(i = shouldSave; i < searchPopup.childNodes.length - (this.isSearchbar ? 2 : 0); i++)
            {
                str.data = searchPopup.childNodes[i].getAttribute("label");
                prefs.setComplexValue(i.toString(), Components.interfaces.nsISupportsString, str);
            }
            for(; prefs.getPrefType(i.toString()) == prefs.PREF_STRING; i++)
            {
                try
                {
                    prefs.clearUserPref(i.toString())
                } catch(e) {}
            }
        }
    }; // checkForRemovedEngines

    /* Function onPopupHidden:
     * Is executed each time the menupopup with the search plugins in it is hidden. Removes all "ordinal"-attributes which are used for alphabetical sorting.
     * Wird jedes mal aufgerufen wenn das Popup-Menü mit den Suchmaschinen versteckt wird. Entfernt alle "ordinal"-Attribute die für die Alphabetische Sortierung benutzt werden.
     */
    this.onPopupHidden = function(aEvent)
    {
    }

    /* Function onPopupShowing:
     * Starts all functions that could be needed when the popup with the search plugins is shown.
     * Startet alle Funktionen, die gebraucht werden könnten, wenn das Menü mit den Suchmaschinen gezeigt wird.
     */
    this.onPopupShowing = function(aEvent)
    {
        this.checkForNewEngines(aEvent);
        this.checkChangedOrder(aEvent);
        this.checkForRemovedEngines(aEvent);
        this.hideAddEngines(aEvent);
        this.addContextMenu(aEvent);
    }; //onPopupShowing

    /* Function addContextMenu:
     * Adds the context menu to the search plugins.
     * Gibt den Seuchmaschinen ihr Kontextmenü.
     */
    this.addContextMenu = function(aEvent)
    {
        // the context menu will cause strange things when used in ContextSearch's popup
        if(this.isSearchbar)
        {
            var elems = aEvent.target.getElementsByAttribute("type", "checkbox");
            for(var i = 0; i < elems.length; i++)
            {
                if(elems[i].tagName == "menuitem")
                    elems[i].setAttribute("context", "seo-main-context")
            }
        }
    }; // addContextMenu

    /* Function contextMenuHidden:
     * Hides the menu with search plugins when the context menu is hidden as otherwise the popup-menu would remain visible until something from the menu is selected.
     * Blendet das Menü mit den Suchmaschine aus, wenn das Kontextmenü ausgeblendet wird, da ersteres sonst nur ausgeblendet werden könnte, wenn etwas aus dem Menü ausgewählt wird.
     */
    this.contextMenuHidden = function(aEvent)
    {
        document.popupNode.parentNode.hidePopup();
    }; // contextMenuHidden

    /* Function alphabetical:
     * Sorts the search engines alphabetically.
     * Sortiert die Suchmaschinen alphabetisch.
     */
    this.alphabetical = function(elems, aChange)
    {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        var seoPrefs = prefs.getBranch("extensions.searchengineordering.");

        var sort = seoPrefs.getIntPref("sort");

        if(typeof elems == "undefined" || elems === null)
        {
            elems = this.mPopup;
        }

        if(typeof aChange != "undefined" && aChange !== null)
        {
            if(aChange != 1 && aChange != 2 && aChange != 0)
                aChange = 0;
            seoPrefs.setIntPref("sort", aChange);
            sort = aChange;
        }

        document.getElementById("seo-sort-alphabetical-az").setAttribute("checked", (sort === 1).toString());
        document.getElementById("seo-sort-alphabetical-za").setAttribute("checked", (sort === 2).toString());
        document.getElementById("seo-sort-manual").setAttribute("checked", (sort !== 1 && sort !== 2).toString());

        if(sort === 1 || sort === 2)
        {
            var labels = new Array();
            for(var i = 0; i < elems.childNodes.length; i++)
            {
                if(elems.childNodes[i].tagName == "menuitem" && elems.childNodes[i].getAttribute("anonid") != "addengine-menuitem")
                    labels.push(elems.childNodes[i].getAttribute("label"));
            }

            labels.sort(function (aFirst, aSecond)
                {
                    if(aFirst == aSecond)
                        return 0;
 
                    while(aFirst.charAt(0) == aSecond.charAt(0)) // cut from the beginning what is the same
                    {
                        aFirst = aFirst.substring(1, aFirst.length);
                        aSecond = aSecond.substring(1, aSecond.length);

                        if(aFirst.length == 1)
                            return -1;
                        if(aSecond.length == 1)
                            return 1;
                    }

                    if(aFirst.charAt(0).toUpperCase() != aSecond.charAt(0).toUpperCase())
                    {
                        aFirst = aFirst.toUpperCase();
                        aSecond = aSecond.toUpperCase();
                    }

                    return aFirst.charCodeAt(0) - aSecond.charCodeAt(0);
                });

            if(sort === 2)
                labels.reverse();

            var str = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            for(i = labels.length - 1; i >= 0; i--)
            {
                str.data = labels[i];
                prefs.setComplexValue("browser.search.order." + (i + 1).toString(), Components.interfaces.nsISupportsString, str);
            }
            labels.unshift(null);
            this.sortPlugins(labels);

            if(elems.lastChild.getAttribute("anonid") == "addengine-menuitem")
            {
                elems.lastChild.style.MozBoxOrdinalGroup = labels.length.toString();
                elems.lastChild.previousSibling.style.MozBoxOrdinalGroup = labels.length.toString();
            }
        }

        return sort;
    };

    /* Function hideAddEngines:
     * Hides the menuitem "Add Engines..." and the menuseparator before it.
     * Versteckt die Option "Suchmaschinen hinzufügen..." und den Separator davor.
     */
    this.hideAddEngines = function(aEvent)
    {
        var searchPopup = aEvent.target;

        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.searchengineordering.");

        var hide = prefs.getBoolPref("addEngines.hide");
        searchPopup.lastChild.hidden = hide; // hide or show the menuitem...
        searchPopup.lastChild.previousSibling.hidden = hide; // ...and the menuseparator
    }; // hideAddEngines

    /* Function onEnginePopupCommand:
     * Overwrites the original method "onEnginePopupCommand" that changes the current search-engine and opens the "Add Engines"-page.
     * Überschreibt die originale Methode "onEnginePopupCommand", die die derzeitige Suchmaschine wechselt und die "Suchmaschine hinzufügen"-Seite aufruft.
     */
    this.onEnginePopupCommand = function(aEvent)
    {
          var target = aEvent.target
          if (target.getAttribute("anonid") == "addengine-menuitem") {
            try {
              var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                    .getService(Components.interfaces.nsIPrefService);
              prefs = prefs.getBranch("extensions.searchengineordering.");
              var URL = prefs.getComplexValue("addEngines.url", Components.interfaces.nsISupportsString).data;
              if(URL == "chrome://searchengineordering/locale/defaults.properties")
                URL = prefs.getComplexValue("addEngines.url",
                                              Components.interfaces.nsIPrefLocalizedString).data;
              loadURI(URL);
            } catch(e) {
              var regionBundle = document.getElementById("bundle_browser_region");
              loadURI(regionBundle.getString("searchEnginesURL"), null, null);
            }
          } else {
            this.currentEngine = target.id;
            if(target.newEngine)
              this.parentNode.setAttribute("src", target.getAttribute("src"));
            this.focus();
            this.select();
          }
        
    };

    /* Function _getPrefOrder:
     * Returns the saved order of the search plugins.
     * Gibt die gespeichert Reihenfolge der Suchmaschinen zurück.
     */
    this._getPrefOrder = function()
    {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("browser.search.order.");
        var children = prefs.getChildList("", {});
        children = children.filter(function(aCurrentValue, aCurrentIndex, aArray)
                                   {
                                       return !isNaN(parseInt(aCurrentValue));
                                   }, this);

        children.sort(function (aFirst, aSecond)
        {
            if(isNaN(parseInt(aFirst)) || aFirst.toString().length > aSecond.toString().length)
                return -1; // aFirst should keep being over aSecond

            if(isNaN(parseInt(aSecond)) || aFirst.toString().length < aSecond.toString().length)
                return 1; // aSecond should be the over aFirst

            if(aFirst == aSecond)
                return 0; // aFirst and aSecond are the same => do nothing

            while(aFirst.toString().charAt(0) == aSecond.toString().charAt(0)) // cut any letters from the beginning
            {
                aFirst = aFirst.substring(1, aFirst.length);
                aSecond = aSecond.substring(1, aSecond.length);
            }

            return - (parseInt(aFirst.toString().charAt(0)) - parseInt(aSecond.toString().charAt(0)));
        });

        var prefOrder = new Array(children.length);
        for(var i = children.length - 1; i >= 0; i--)
        {
            try
            {
                if(prefs.getPrefType(parseInt(children[i])) == prefs.PREF_STRING)
                    prefOrder.push(prefs.getComplexValue(children[i], Components.interfaces.nsISupportsString).data);
            } catch(e) { continue; }

            // if the order is not already changed you get this result
            if(prefOrder[prefOrder.length - 1] == "chrome://browser-region/locale/region.properties")
                prefOrder[prefOrder.length - 1] = prefs.getComplexValue(children[i],
                                                  Components.interfaces.nsIPrefLocalizedString).data;
                // this returns Google (children[i] = 1) or Yahoo (children[i] = 2)
        }

        return prefOrder;
    }; // _getPrefOrder

    /* function openSeo()
     * Starts Search Engine Ordering in a new window or tab.
     * Startet Search Engine Ordering in einem neuen Fenster oder Tab, je nachdem, was der Benutzer eingestellt hat.
     */
    this.openSeo = function()
    {
        const URL = "chrome://searchengineordering/content/order.xul";

        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                       .getService(Components.interfaces.nsIWindowMediator);
        var seoWindow = windowMediator.getMostRecentWindow("seo:window"); // get topmost seo-window
        if(seoWindow != null) // is there already an opened seo-window
            seoWindow.focus(); // give it the focus
        else
        {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                  .getService(Components.interfaces.nsIPrefService);
            prefs = prefs.getBranch("extensions.searchengineordering.");

            var foundTab = function(aURL)
            {
                var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                               .getService(Components.interfaces.nsIWindowMediator); 
                var windowEnumerator = windowMediator.getEnumerator("navigator:browser");
                var win, i, tab, found = false;
                while(windowEnumerator.hasMoreElements() && found === false)
                {
                    win = windowEnumerator.getNext();
                    for(i = 0; i < win.gBrowser.mPanelContainer.childNodes.length && found === false; i++)
                    {
                        tab = win.gBrowser.getBrowserAtIndex(i);
                        if(tab.currentURI.spec == aURL)
                        {
                            found = new Array();
                            found["win"] = win;
                            found["tabindex"] = i;
                        }
                    }
                }

                return found;
            } (URL);
            if(foundTab != false)
            {
                foundTab["win"].gBrowser.selectedTab = foundTab["win"].gBrowser.mTabs[foundTab["tabindex"]];
                foundTab["win"].focus();
            }
            else
            {
                if(prefs.getBoolPref("options.openintab"))
                {
                    if(gBrowser.selectedBrowser.currentURI.spec == "about:blank")
                    {
                        gBrowser.selectedBrowser.loadURI(URL);
                    }
                    else
                        gBrowser.selectedTab = gBrowser.addTab(URL);
                }
                else
                {
                    window.openDialog(URL, "seooptions", "chrome,resizable,dependent,dialog,minimizable=0,centerscreen,alwaysRaised=0"); // open a new window
                }
            } // no opened tab with seo in it
        } // no opened window with only seo in it

        return true;
        
    }; // openSeo

    /* function removeEngine:
     * Deletes a search-plugin.
     * Löscht ein Search-Plugin.
     */
    this.removeEngine = function(item, aWin)
    {
        if(item.className.indexOf("searchplugin-item") == -1)
            return -1;

        var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                        .getService(Components.interfaces.nsILocaleService)
                        .getApplicationLocale();
        var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                      .getService(Components.interfaces.nsIStringBundleService)
                      .createBundle("chrome://searchengineordering/locale/global.properties", appLocale);

        if(!strings)
            return -2;

        var itemName = item.getAttribute("label");

        var itemLocations = Array();
        itemLocations.push(("file:/" + item.getAttribute("id").substring(7, item.getAttribute("id").length).replace(/\%5C/g, "/")).replace(/^file\:\/{4}([^\/])/, "file:///$1"));
        if(item.getAttribute("src") != "")
            itemLocations.push(item.getAttribute("src"));
        else
            itemLocations.push(null);

        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.searchengineordering.");

        var remove = true;
        if(prefs.getBoolPref("remove.ask"))
        {
            var askAgain = {value:false};
            remove = prompts.confirmEx(aWin,
                                       strings.GetStringFromName("name"),
                                       strings.GetStringFromName("deleteConfirm1") + itemName + strings.GetStringFromName("deleteConfirm2"),
                                       (1 * 3) + (256 * 4), "3", "4", "",
                                       strings.GetStringFromName("dontAskAgain"), askAgain) == 0;
            if(askAgain.value == true && remove)
                prefs.setBoolPref("remove.ask", false);
        }

        if(remove)
        {
            var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                     .createInstance(Components.interfaces.nsIFileProtocolHandler);
            var searchPlugin = url2file.getFileFromURLSpec(itemLocations[0]);
            if(itemLocations[1] !== null)
            {
                var image = url2file.getFileFromURLSpec(itemLocations[1]);
            }
            else
                var image = false;

            if(!searchPlugin.exists() || (image !== false && !image.exists()))
                prompts.alert(window, strings.GetStringFromName("name"), strings.GetStringFromName("deleteNotExisting"))

            if(image !== false && searchPlugin.exists() && image.exists() && !searchPlugin.isWritable() && !image.isWritable())
            {
                prompts.alert(aWin, strings.GetStringFromName("name"),
                              strings.GetStringFromName("deleteNoRightsBegin")
                               + searchPlugin.path
                               + strings.GetStringFromName("deleteNoRightsAnd")
                               + image.path
                               + strings.GetStringFromName("deleteNoRightsEnd"));
                return -3;
            }
            else
            {
                if(searchPlugin.exists() && !searchPlugin.isWritable())
                {
                    prompts.alert(aWin, strings.GetStringFromName("name"),
                                  strings.GetStringFromName("deleteNoRightsBegin")
                                   + searchPlugin.path
                                   + strings.GetStringFromName("deleteNoRightsEnd"));
                    var shouldReturn = -0.5;
                }
                if(image !== false && image.exists() && !image.isWritable())
                {
                    prompts.alert(aWin, strings.GetStringFromName("name"),
                                  strings.GetStringFromName("deleteNoRightsBegin")
                                   + image.path
                                   + strings.GetStringFromName("deleteNoRightsEnd"));
                    var shouldReturn = -0.5;
                }
            }

            if(searchPlugin.exists() && searchPlugin.isWritable())
            {
                searchPlugin.remove(false);
            }
            if(image !== false && image.exists() && image.isWritable())
            {
                if(image.path.substring(0, image.path.lastIndexOf(".")) + ".src" == searchPlugin.path)
                    image.remove(false);
            }
            item.parentNode.removeChild(item);

            if(shouldReturn)
                return shouldReturn;
            else
                return 1;
        }

        return -4;
    }; // removeEngine

    /* function createSearchplugin:
     * Opens the dialog that creates a new search-plugin and collects the needed data.
     * Öffnet den Dialog zum Erstellen eines Search-Plugins und sammelt die benötigten Daten.
     */
    this.createSearchplugin = function(aFormitem, aEvent, aAllData)
    {
        function _getFaviconFromURL(aUrl)
        {
            var favicon = "chrome://searchengineordering/skin/default-engine.png";
            if(gBrowser.selectedBrowser.currentURI.spec == aUrl)
            {
                if(gBrowser.selectedTab.hasAttribute("image"))
                    favicon = gBrowser.selectedTab.getAttribute("image");
            }
            else
            {
                for(var i = 0; i < gBrowser.browsers.length; i++)
                {
                    if(gBrowser.browsers[i].currentURI.spec == aUrl)
                    {
                        if(gBrowser.tabs[i].getAttribute("image") != "")
                            favicon = gBrowser.tabs[i].getAttribute("image");
                        break;
                    }
                }
            }
            return favicon;
        }

        if(typeof aAllData != "undefined" || aFormitem.form.method.toLowerCase() == "get" || aFormitem.form.method == "")
        {
            if(typeof aAllData == "undefined")
            {
                var title = aFormitem.ownerDocument.title;
                var url = aFormitem.ownerDocument.location.href;
                var favicon = _getFaviconFromURL(url);
                var charset = aFormitem.ownerDocument.characterSet;
            }
            else
            {
                var title = aAllData.title;
                var url = aAllData.url;
                var favicon = aAllData.favicon;
                var charset = aAllData.charset;
            }
            window.openDialog("chrome://searchengineordering/content/createSearchplugin.xul",
                              "seoCreateSearchplugin", "chrome,resizable,dependent,dialog,minimizable=0,alwaysRaised",
                              aFormitem, title, url, favicon, charset);
        }
        else // no get-form, try it and ask if it worked
        {
            var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                                      .getService(Components.interfaces.nsILocaleService)
                                      .getApplicationLocale();
            var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                    .getService(Components.interfaces.nsIStringBundleService)
                                    .createBundle("chrome://searchengineordering/locale/createPostGet.properties", appLocale);

            var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                    .getService(Components.interfaces.nsIPromptService);

            if(!prompts.confirm(window, strings.GetStringFromName("postget-title"), strings.GetStringFromName("postget-info")))
                return;

            if(aFormitem.value == aFormitem.defaultValue)
            {
                var testvalue = {value:""};
                while(testvalue.value == "")
                {
                   if(!prompts.prompt(window, strings.GetStringFromName("postget-title"), strings.GetStringFromName("postget-testvalue"), testvalue, null, {value:false}))
                        return;
                }
                aFormitem.value = testvalue.value;
            }

            var allData = {};
            allData.title = aFormitem.ownerDocument.title;
            allData.url = aFormitem.ownerDocument.location.href;
            allData.favicon = _getFaviconFromURL(allData.url);
            allData.charset = aFormitem.ownerDocument.characterSet;

            function loader(aAnotherEvent)
            {
                aAnotherEvent.currentTarget.removeEventListener("load", loader, true);

                //window.setTimeout(function() {
                    if(prompts.confirmEx(window, strings.GetStringFromName("postget-title"), strings.GetStringFromName("postget-worked"), 1*3 + 256*4, "yes", "no", "", null, {value:false}) == 0)
                    {
                        seo.createSearchplugin(aFormitem, aEvent, allData);
                    }
                //}, 50);
            }
            gBrowser.selectedBrowser.addEventListener("load", loader, true, true);

            aFormitem.form.method = "get";
            aFormitem.form.submit();
            
        }
    }; // createSearchplugin

    this.newPluginsObserver = new function()
    {
        this.registerObserver = function()
        {
            if(this._branch) return;

            var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);
            this._branch = prefService.getBranch("extensions.searchengineordering.create.");
            this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
            this._branch.addObserver("", this, false);

            this.observe(this._branch, null, "newengines");
        };

        this.unregisterObserver = function()
        {
            if(!this._branch)
                return;

            this._branch.removeObserver("", this);
            this._branch = false;
        };

        this.observe = function(aSubject, aTopic, aData)
        {
            // aSubject is the nsIPrefBranch we're observing
            // aData is the name of the pref that's been changed (relative to aSubject)
            if(typeof Components == "undefined")
            {
                this.unregisterObserver();
                return;
            }

            if(aData == "newengines")
            {
                var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                            .getService(Components.interfaces.nsIPrefService);
                var branch = prefService.getBranch("extensions.searchengineordering.create.");

                var rawData = branch.getComplexValue(aData, Components.interfaces.nsISupportsString).data;
                if(rawData == "")
                    return;

                var newEngines = rawData.split("|");
                var raw = Components.classes["@mozilla.org/supports-string;1"]
                                    .createInstance(Components.interfaces.nsISupportsString);
                raw.data = "";
                var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                         .createInstance(Components.interfaces.nsIFileProtocolHandler);
                var file, node;
                for(var i = 0; i < newEngines.length; i++)
                {
                    newEngines[i] = decodeURIComponent(newEngines[i]).split("^");
                    if(newEngines[i].length < 3)
                        continue;

                    if(this.seo.mPopup.getElementsByAttribute("id", newEngines[i][0])[0] == undefined)
                    {
                        file = url2file.getFileFromURLSpec(newEngines[i][1].replace(/.(?:png|gif|jpe?g)$/, ".src"));
                        if(file.exists())
                        {
                            node = this.seo.mPopup.childNodes[1].cloneNode(true);
                            node.setAttribute("value", newEngines[i][0]);
                            node.setAttribute("id",    newEngines[i][0]);
                            node.setAttribute("src",   newEngines[i][1]);
                            node.setAttribute("label", newEngines[i][2]);
                            node.newEngine = true;
                            this.seo.mPopup.appendChild(node);

                            raw.data = raw.data + "|" + encodeURIComponent(newEngines[i][0]) + "^" + encodeURIComponent(newEngines[i][1]) + "^" + encodeURIComponent(newEngines[i][2]);
                        }
                    }
                    else
                    {
                        if(this.seo.mPopup.getElementsByAttribute("id", newEngines[i][0])[0].newEngine)
                        {
                            raw.data = raw.data + "|" + encodeURIComponent(newEngines[i][0]) + "^" + encodeURIComponent(newEngines[i][1]) + "^" + encodeURIComponent(newEngines[i][2]);
                        }
                    }
                }
                raw.data = raw.data.substring(1, raw.data.length);
                if(raw.data != rawData)
                    branch.setComplexValue(aData, Components.interfaces.nsISupportsString, raw);
            }
        };
    }();

    this.onContextMenuShowing = function(aEvent)
    {
        gContextMenu.showItem("seo-create-searchplugin", gContextMenu.onTextInput || gContextMenu.onKeywordField);
    }; // onContextMenuShowing
} // seoObj
var seo = new seoObj();

window.addEventListener("load", function() { seo.init() }, false);