var seo = new (function()
{
    this.onLoad = function()
    {
        if(!gDialog.list)
            gDialog.list = document.getElementById("searchEngineList");

        // override Add Engines:
        gDialog.list.removeEventListener("ValueChange", window.onEnginesListValueChange, false);
        window.__origOnEnginesListValueChange = window.onEnginesListValueChange;
        window.onEnginesListValueChange = seo.onEnginesListValueChange;
        gDialog.list.addEventListener("ValueChange", window.onEnginesListValueChange, false);

        // don't know why this is necessary, otherwise it wouldn't be found when using getElementById or the context-attribute
        document.getElementsByAttribute("id", "seoContext")[0].setAttribute("id", "seoContext"); 
        document.getElementsByAttribute("id", "seo-sort-manual")[0].setAttribute("id", "seo-sort-manual");
        document.getElementsByAttribute("id", "seo-sort-alphabetical-az")[0].setAttribute("id", "seo-sort-alphabetical-az");
        document.getElementsByAttribute("id", "seo-sort-alphabetical-za")[0].setAttribute("id", "seo-sort-alphabetical-za");

        _removeDeletedEngines();
        _addContextMenu();

        seo.sorting.register(true);
        seo.alphabet.registerObserver();
        seo.alphabet._updateChecked();
        seo.alphabet._do();

        gDialog.list.addEventListener("popupshowing", seo.onEnginePopupShowing, false);
        gDialog.list.addEventListener("popuphding", seo.onEnginePopupHiding, false);

        window.addEventListener("unload", seo.sorting.unregister, false);
        window.addEventListener("unload", seo.alphabet.unregisterObserver, false);

        
        var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                        .getService(Components.interfaces.nsILocaleService)
                        .getApplicationLocale();
        var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                      .getService(Components.interfaces.nsIStringBundleService)
                      .createBundle("chrome://searchengineordering/locale/searchDialog.properties", appLocale);
        var extra2 = document.getAnonymousElementByAttribute(document.documentElement, "dlgtype", "extra2");
        extra2.label = strings.GetStringFromName("sort-button");
        extra2.hidden = false;
        extra2.nextSibling.hidden = false;
        extra2.addEventListener("command", function(aEvent)
            {
                aEvent.stopPropagation();
                aEvent.preventDefault();
                window.openDialog("chrome://searchengineordering/content/order.xul", "seooptions", "chrome,resizable,dependent,dialog,minimizable=0,alwaysRaised,modal,centerscreen");
                return;
            }, true);
        return;
    };
    this.onEnginePopupShowing = function()
    {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.searchengineordering.");
        var hide = prefs.getBoolPref("addEngines.hide");
        var menuitem = document.getElementById("addenginemenuitem");
        var separator = document.getElementById("addenginesep");
        if(menuitem)
            menuitem.hidden = hide;
        if(separator)
            separator.hidden = hide;

        this.enginePopupHidden = false;

        _removeDeletedEngines();
        _addContextMenu();
    };
    this.onEnginePopupHiding = function()
    {
        this.enginePopupHidden = true;
    };
    this.sorting = new(function()
    {
        this.register = function(aUpdate)
        {
            if(this._branch) return;

            var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);
            this._branch = prefService.getBranch("browser.search.order.");
            this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
            this._branch.addObserver("", this, false);

            if(aUpdate)
                this._updateEnginesOrder();
        }
        this.unregister = function()
        {
            if(!this._branch) return;
            this._branch.removeObserver("", this);
            this._branch = false;
        }
        this.observe = function(aSubject, aTopic, aData)
        {
            // aSubject is the nsIPrefBranch we're observing
            // aData is the name of the pref that's been changed (relative to aSubject)
            if(aTopic != "nsPref:changed") return;
            if(isNaN(parseInt(aData))) return;
            if(typeof Components == "undefined") return;

            this._updateEnginesOrder();
        }
        this._updateEnginesOrder = function()
        {
            var prefOrder = _getPrefOrder();
            var menu = gDialog.list.menupopup;
            for(var i = 0; i < prefOrder.length; i++)
            {
                if(menu.getElementsByAttribute('label', prefOrder[i]).length >= 1) // perhaps there is no menuitem with this label (e.g. because of old prefs)
                {
                    menu.insertBefore(menu.getElementsByAttribute('label', prefOrder[i])[0], menu.childNodes[i]);
                }
            }
        }
        this.changeTo = function(aNewOrder)
        {
            // sort the popup only one time
            this._realUpdateEnginesOrder = this._updateEnginesOrder;
            this._updateEnginesOrder = function() { /* does nothing until the update-batch has completed */ };

            var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);
            var prefBranch = prefService.getBranch("browser.search.order.");
            var str = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);

            var children = prefBranch.getChildList("", {});
            children = children.filter(function(aCurrentValue, aCurrentIndex, aArray)
                                       {
                                           return !isNaN(parseInt(aCurrentValue));
                                       }, this);
            for(var i in children)
            {
                try {
                    if(aNewOrder[i] != "undefined")
                        prefBranch.clearUserPref(children[i]);
                } catch(e) { }
            }

            for(i = 0; i < aNewOrder.length; i++)
            {
                if(typeof aNewOrder[i] == "undefined")
                    continue;

                str.data = aNewOrder[i];
                prefBranch.setComplexValue((i + 1).toString(), Components.interfaces.nsISupportsString, str);
            }

            this._updateEnginesOrder = this._realUpdateEnginesOrder;
            this._realUpdateEnginesOrder = null;
            this._updateEnginesOrder();
        }
    })();

    this.alphabet = new (function()
    {
        this.changeTo = function(aDir)
        {
            var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);
            var extBranch = prefService.getBranch("extensions.searchengineordering.");

            if(aDir !== 0 && aDir !== 1 && aDir !== 2)
                aDir = 0;

            if(extBranch.getIntPref("sort") != aDir)
                extBranch.setIntPref("sort", aDir);
        };

        this.registerObserver = function()
        {
            if(this._branch) return;

            var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);
            this._branch = prefService.getBranch("extensions.searchengineordering.");
            this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
            this._branch.addObserver("", this, false);
        };

        this.unregisterObserver = function()
        {
            if(!this._branch) return;

            this._branch.removeObserver("", this);
            this._branch = false;
        };

        this.observe = function(aSubject, aTopic, aData)
        {
            // aSubject is the nsIPrefBranch we're observing
            // aData is the name of the pref that's been changed (relative to aSubject)
            if(typeof Components == "undefined")
            {
                this.unregisterObserver();
                return;
            }
            if(aData == "sort")
            {
                const PREFSERVICE = Components.classes["@mozilla.org/preferences-service;1"]
                                              .getService(Components.interfaces.nsIPrefService);
                const EXTBRANCH = PREFSERVICE.getBranch("extensions.searchengineordering.");

                this._dir = EXTBRANCH.getIntPref("sort");

                this._updateChecked();
                this._do();
            }
        };

        this._updateChecked = function()
        {
            if(!this._dir)
            {
                var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                              .getService(Components.interfaces.nsIPrefService);
                const extBranch = prefService.getBranch("extensions.searchengineordering.");

                this._dir = extBranch.getIntPref("sort");
            }

            document.getElementById("seo-sort-manual")         .setAttribute("checked", (this._dir !== 1 && this._dir !== 2).toString());
            document.getElementById("seo-sort-alphabetical-az").setAttribute("checked", (this._dir === 1)                   .toString());
            document.getElementById("seo-sort-alphabetical-za").setAttribute("checked", (this._dir === 2)                   .toString());
        };

        this._do = function()
        {
            if(this._dir == 1 || this._dir == 2)
            {
                var engineNames = new Array();
                var maxlength = gDialog.list.menupopup.childNodes.length;

                if(document.getElementById("addenginemenuitem") != null)
                    maxlength = maxlength - 1;
                if(document.getElementById("addenginesep") != null)
                    maxlength = maxlength - 1;

                for(var i = 0; i < maxlength; i++)
                {
                    if(gDialog.list.menupopup.childNodes[i].tagName != "template")
                        engineNames[i] = gDialog.list.menupopup.childNodes[i].getAttribute("label");
                }
                engineNames.filter(function(aValue) { return aValue != undefined; });
                engineNames.sort(function(aFirst, aSecond)
                    {
                        if(aFirst == aSecond)
                            return 0;
 
                        while(aFirst.charAt(0) == aSecond.charAt(0)) // cut from the beginning what is the same
                        {
                            aFirst = aFirst.substring(1, aFirst.length);
                            aSecond = aSecond.substring(1, aSecond.length);

                            if(aFirst.length == 1)
                                return -1;
                            if(aSecond.length == 1)
                                return 1;
                        }

                        if(aFirst.charAt(0).toUpperCase() != aSecond.charAt(0).toUpperCase())
                        {
                            aFirst = aFirst.toUpperCase();
                            aSecond = aSecond.toUpperCase();
                        }

                        return aFirst.charCodeAt(0) - aSecond.charCodeAt(0);
                    });

                if(this._dir == 2)
                    engineNames.reverse();

                seo.sorting.changeTo(engineNames);
            }
        };
    })();

    function _addContextMenu()
    {
        gDialog.list.menupopup.getElementsByTagName("template")[0].firstChild.setAttribute("context", "seoContext"); // the menuitem in the template
        for(var i = 0; i < gDialog.list.menupopup.childNodes.length; i++)
        {
            if(gDialog.list.menupopup.childNodes[i].tagName == "menuitem" && gDialog.list.menupopup.childNodes[i].id != "addenginemenuitem")
                gDialog.list.menupopup.childNodes[i].setAttribute("context", "seoContext");
        }
    }

    /* Function _getPrefOrder:
     * Returns the saved order of the search plugins.
     * Gibt die gespeichert Reihenfolge der Suchmaschinen zurück.
     */
    function _getPrefOrder()
    {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("browser.search.order.");
        var children = prefs.getChildList("", {});
        children = children.filter(function(aCurrentValue, aCurrentIndex, aArray)
                                   {
                                       return !isNaN(parseInt(aCurrentValue));
                                   }, this);

        children.sort(function (aFirst, aSecond)
        {
            if(isNaN(parseInt(aFirst)) || aFirst.toString().length > aSecond.toString().length)
                return -1; // aFirst should keep being over aSecond

            if(isNaN(parseInt(aSecond)) || aFirst.toString().length < aSecond.toString().length)
                return 1; // aSecond should be the over aFirst

            if(aFirst == aSecond)
                return 0; // aFirst and aSecond are the same => do nothing

            while(aFirst.toString().charAt(0) == aSecond.toString().charAt(0)) // cut any letters from the beginning
            {
                aFirst = aFirst.substring(1, aFirst.length);
                aSecond = aSecond.substring(1, aSecond.length);
            }

            return - (parseInt(aFirst.toString().charAt(0)) - parseInt(aSecond.toString().charAt(0)));
        });

        var oldChildren = children;
        children = new Array();
        for(var i in oldChildren)
        {
            children.push(oldChildren[i]);
        }

        var prefOrder = new Array();
        for(i = children.length - 1; i >= 0; i--)
        {
            try
            {
                if(prefs.getPrefType(parseInt(children[i])) == prefs.PREF_STRING)
                    prefOrder.push(prefs.getComplexValue(children[i], Components.interfaces.nsISupportsString).data);
            } catch(e) { continue; }

            // if the order is not already changed you get this result
            if(prefOrder[prefOrder.length - 1] == "chrome://browser-region/locale/region.properties")
                prefOrder[prefOrder.length - 1] = prefs.getComplexValue(children[i],
                                                  Components.interfaces.nsIPrefLocalizedString).data;
                // this returns Google (children[i] = 1) or Yahoo (children[i] = 2)
        }

        return prefOrder;
    }; // _getPrefOrder

    this.onEnginesListValueChange = function()
    {
      if (gDialog.list.value == "addengines") {
        try
        {
          var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
          prefs = prefs.getBranch("extensions.searchengineordering.");
          var url = prefs.getComplexValue("addEngines.url", Components.interfaces.nsISupportsString).data;
          if(url == "chrome://searchengineordering/locale/defaults.properties")
            url = prefs.getComplexValue("addEngines.url",
                                           Components.interfaces.nsIPrefLocalizedString).data;
        } catch(e) {
          var url = gDialog.browserbundle.getString("searchEnginesURL");
        }
        window.opener.openNewWindowWith(url, null, false);
        gDialog.list.selectedIndex = gSelectedEngineIndex;
      } else {
        gSelectedEngineIndex = gDialog.list.selectedIndex;
      }
    };

    // remove engines that do not exist
    function _removeDeletedEngines()
    {
        var url2file = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                                 .createInstance(Components.interfaces.nsIFileProtocolHandler);
        var plugin, img;
        for(var i = 0; i < gDialog.list.firstChild.childNodes.length; i++)
        {
            if(gDialog.list.firstChild.childNodes[i].tagName == "menuitem")
            {
                plugin = gDialog.list.firstChild.childNodes[i].getAttribute("id").replace(/^engine:/, "file:/");
                if(plugin == "addenginemenuitem")
                    continue;

                plugin = plugin.replace(/\%5C/g, "/");
                plugin = url2file.getFileFromURLSpec(plugin);

                if(!plugin.exists())
                {
                    gDialog.list.firstChild.removeChild(gDialog.list.firstChild.childNodes[i]);

                    try
                    {
                        img = url2file.getFileFromURLSpec(gDialog.list.firstChild.childNodes[i].getAttribute("src"));
                        if(img.exists() && img.isWritable())
                            img.remove(false);
                    }
                    catch(e) { }
                }
            }
        }
    }

    this.deleteEngine = function(aEngineMenuitem)
    {
        aEngineMenuitem.parentNode.hidePopup();
        aEngineMenuitem.className = "searchplugin-item";
        opener.seo.removeEngine(aEngineMenuitem, window);
        aEngineMenuitem.className = "";
    };
})();
window.addEventListener("load", seo.onLoad, false);
